/**
* DO NOT EDIT THIS FILE.
* See the following change record for more information,
* https://www.drupal.org/node/2815083
* @preserve
**/

(function ($, Drupal, _, document) {
  if (Drupal.filterConfiguration) {
    Drupal.filterConfiguration.liveSettingParsers.filter_html = {
      getRules() {
        const currentValue = document.querySelector('#edit-filters-filter-html-settings-allowed-html').value;

        const rules = Drupal.behaviors.filterFilterHtmlUpdating._parseSetting(currentValue);

        const rule = new Drupal.FilterHTMLRule();
        rule.restrictedTags.tags = ['*'];
        rule.restrictedTags.forbidden.attributes = ['style', 'on*'];
        rules.push(rule);
        return rules;
      }

    };
  }

  Drupal.behaviors.filterFilterHtmlUpdating = {
    $allowedHTMLFormItem: null,
    $allowedHTMLDescription: null,
    userTags: {},
    autoTags: null,
    newFeatures: {},

    attach(context, settings) {
      const that = this;
      once('filter-filter_html-updating', '[name="filters[filter_html][settings][allowed_html]"]', context).forEach(formItem => {
        that.$allowedHTMLFormItem = $(formItem);
        that.$allowedHTMLDescription = that.$allowedHTMLFormItem.closest('.js-form-item').find('.description');
        that.userTags = that._parseSetting(formItem.value);
        $(document).on('drupalEditorFeatureAdded', (e, feature) => {
          that.newFeatures[feature.name] = feature.rules;

          that._updateAllowedTags();
        }).on('drupalEditorFeatureModified', (e, feature) => {
          if (that.newFeatures.hasOwnProperty(feature.name)) {
            that.newFeatures[feature.name] = feature.rules;

            that._updateAllowedTags();
          }
        }).on('drupalEditorFeatureRemoved', (e, feature) => {
          if (that.newFeatures.hasOwnProperty(feature.name)) {
            delete that.newFeatures[feature.name];

            that._updateAllowedTags();
          }
        });
        that.$allowedHTMLFormItem.on('change.updateUserTags', function () {
          that.userTags = _.difference(that._parseSetting(this.value), that.autoTags);
        });
      });
    },

    _updateAllowedTags() {
      this.autoTags = this._calculateAutoAllowedTags(this.userTags, this.newFeatures);
      this.$allowedHTMLDescription.find('.editor-update-message').remove();

      if (!_.isEmpty(this.autoTags)) {
        this.$allowedHTMLDescription.append(Drupal.theme('filterFilterHTMLUpdateMessage', this.autoTags));

        const userTagsWithoutOverrides = _.omit(this.userTags, _.keys(this.autoTags));

        this.$allowedHTMLFormItem.val(`${this._generateSetting(userTagsWithoutOverrides)} ${this._generateSetting(this.autoTags)}`);
      } else {
        this.$allowedHTMLFormItem.val(this._generateSetting(this.userTags));
      }
    },

    _calculateAutoAllowedTags(userAllowedTags, newFeatures) {
      const editorRequiredTags = {};
      Object.keys(newFeatures || {}).forEach(featureName => {
        const feature = newFeatures[featureName];
        let featureRule;
        let filterRule;
        let tag;

        for (let f = 0; f < feature.length; f++) {
          featureRule = feature[f];

          for (let t = 0; t < featureRule.required.tags.length; t++) {
            tag = featureRule.required.tags[t];

            if (!_.has(editorRequiredTags, tag)) {
              filterRule = new Drupal.FilterHTMLRule();
              filterRule.restrictedTags.tags = [tag];
              filterRule.restrictedTags.allowed.attributes = featureRule.required.attributes.slice(0);
              filterRule.restrictedTags.allowed.classes = featureRule.required.classes.slice(0);
              editorRequiredTags[tag] = filterRule;
            } else {
              filterRule = editorRequiredTags[tag];
              filterRule.restrictedTags.allowed.attributes = _.union(filterRule.restrictedTags.allowed.attributes, featureRule.required.attributes);
              filterRule.restrictedTags.allowed.classes = _.union(filterRule.restrictedTags.allowed.classes, featureRule.required.classes);
            }
          }
        }
      });
      const autoAllowedTags = {};
      Object.keys(editorRequiredTags).forEach(tag => {
        if (!_.has(userAllowedTags, tag)) {
          autoAllowedTags[tag] = editorRequiredTags[tag];
        } else {
          const requiredAttributes = editorRequiredTags[tag].restrictedTags.allowed.attributes;
          const allowedAttributes = userAllowedTags[tag].restrictedTags.allowed.attributes;

          const needsAdditionalAttributes = requiredAttributes.length && _.difference(requiredAttributes, allowedAttributes).length;

          const requiredClasses = editorRequiredTags[tag].restrictedTags.allowed.classes;
          const allowedClasses = userAllowedTags[tag].restrictedTags.allowed.classes;

          const needsAdditionalClasses = requiredClasses.length && _.difference(requiredClasses, allowedClasses).length;

          if (needsAdditionalAttributes || needsAdditionalClasses) {
            autoAllowedTags[tag] = userAllowedTags[tag].clone();
          }

          if (needsAdditionalAttributes) {
            autoAllowedTags[tag].restrictedTags.allowed.attributes = _.union(allowedAttributes, requiredAttributes);
          }

          if (needsAdditionalClasses) {
            autoAllowedTags[tag].restrictedTags.allowed.classes = _.union(allowedClasses, requiredClasses);
          }
        }
      });
      return autoAllowedTags;
    },

    _parseSetting(setting) {
      let tag;
      let rule;
      let attributes;
      let attribute;
      const allowedTags = setting.match(/(<[^>]+>)/g);
      const rules = {};

      for (let t = 0; t < allowedTags.length; t++) {
        const $tagObject = $(allowedTags[t]);
        tag = $tagObject.prop('tagName').toLowerCase();
        rule = new Drupal.FilterHTMLRule();
        rule.restrictedTags.tags = [tag];
        attributes = $tagObject.prop('attributes');

        for (let i = 0; i < attributes.length; i++) {
          attribute = attributes.item(i);
          const attributeName = attribute.nodeName;

          if (attributeName === 'class') {
            const attributeValue = attribute.textContent;
            rule.restrictedTags.allowed.classes = attributeValue.split(' ');
          } else {
            rule.restrictedTags.allowed.attributes.push(attributeName);
          }
        }

        rules[tag] = rule;
      }

      return rules;
    },

    _generateSetting(tags) {
      return _.reduce(tags, (setting, rule, tag) => {
        if (setting.length) {
          setting += ' ';
        }

        setting += `<${tag}`;

        if (rule.restrictedTags.allowed.attributes.length) {
          setting += ` ${rule.restrictedTags.allowed.attributes.join(' ')}`;
        }

        if (rule.restrictedTags.allowed.classes.length) {
          setting += ` class="${rule.restrictedTags.allowed.classes.join(' ')}"`;
        }

        setting += '>';
        return setting;
      }, '');
    }

  };

  Drupal.theme.filterFilterHTMLUpdateMessage = function (tags) {
    let html = '';

    const tagList = Drupal.behaviors.filterFilterHtmlUpdating._generateSetting(tags);

    html += '<p class="editor-update-message">';
    html += Drupal.t('Based on the text editor configuration, these tags have automatically been added: <strong>@tag-list</strong>.', {
      '@tag-list': tagList
    });
    html += '</p>';
    return html;
  };
})(jQuery, Drupal, _, document);